;**************************************************************
;* lib_score.h	Routines to handle scores
;*
;* Copyright (C) 2015,2016	Paul Daniels
;*
;* This program is free software: you can redistribute it and/
;* or modify it under the terms of the GNU General Public
;* License as published by the Free Software Foundation, either
;* version 3 of the License, or (at your option) any later
;* version.
;*
;* This program is distributed in the hope that it will be
;* useful, but WITHOUT ANY WARRANTY; without even the implied
;* warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
;* PURPOSE.  See the GNU General Public License for more
;* details.
;*
;* You should have received a copy of the GNU General Public
;* License along with this program. If not, see
;* <http://www.gnu.org/licenses/>.
;**************************************************************

		.IFNDEF LIB_SCORE_H

		.EQU LIB_SCORE_H	1

; Name:		SCO_RESET
; Function:	Zero score of 4 bytes (8 BCD digits)
; On Entry:	HL = address of score
; On Exit:	None
; Destroys:	A, HL

; Name:		SCO_ADD
; Function:	Add BC (in BCD format) to score
; On Entry:	BC = Value to add. HL = address of score
; On Exit:	None
; Destroys:	A, HL

; Name:		SCO_PRT
; Function:	Output score
; On Entry:	A = digits to print. HL = address of score
; On Exit:	None
; Destroys:	A, BC, HL

; Name:		SCO_HISC
; Function:	Add score to a high score table
; On Entry:	B = No of hi-score entries. DE = address of hi-score.
;		HL = address of score
; On Exit:	A = hi-score position or 0.
; Destroys:	

.MACRO SCO_INIT
		ld hl, \1			; Score
		call SCO_RESET
.ENDM

.MACRO SCO_PRINT
		ld a, \1			; Digits to print
		ld hl, \2			; Score
		call SCO_PRT
.ENDM

.MACRO SCO_ADDVAL
		ld hl, \1			; Score
		ld bc, \2			; Value to add
		call SCO_ADD
.ENDM

.MACRO SCO_ADDHI
		ld hl, \1			; Score
		ld de, \2			; Hi Score table
		ld b, \3			; Entries in table
		call SCO_HISC
.ENDM

		.ENDIF
