;**************************************************************
;* lib_ti9929.h		TI9929 VDP Definitions Header
;*
;* Copyright (C) 2015,2016	Paul Daniels
;*
;* This program is free software: you can redistribute it and/
;* or modify it under the terms of the GNU General Public
;* License as published by the Free Software Foundation, either
;* version 3 of the License, or (at your option) any later
;* version.
;*
;* This program is distributed in the hope that it will be
;* useful, but WITHOUT ANY WARRANTY; without even the implied
;* warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
;* PURPOSE.  See the GNU General Public License for more
;* details.
;*
;* You should have received a copy of the GNU General Public
;* License along with this program. If not, see
;* <http://www.gnu.org/licenses/>.
;**************************************************************

;**************************************************************
; lib_ti9929.h	Defines used by the TI9929 library

		.IFNDEF LIB_TI9929_H

		.EQU LIB_TI9929_H	1

		.EQU VDP_PORT0	1
		.EQU VDP_PORT1	2

;**************************************************************
; VDP table ptrs

		.EQU VDP_SPRMAG_OFF	$00
		.EQU VDP_SPRMAG_ON	$01
		.EQU VDP_SPRSIZ_OFF	$00
		.EQU VDP_SPRSIZ_ON	$02

		; VDP_GRTYPE
		.EQU VDP_GRTEXT32	$00	; 32 x 24
		.EQU VDP_GRTEXT40	$01	; 40 x 24
		.EQU VDP_GRMULTI	$02	; 64 x 48
		.EQU VDP_GRHIRES	$04	; 256 x 192

		; VDP_GRMODE
		.EQU VDP_CHARSET1	$00
		.EQU VDP_COLSET1	$9F

		; 1st and 3rd charset the same, 2nd different

		.EQU VDP_CHARSET1B	$01
		.EQU VDP_COLSET1B	$BF

		; 1st and 2nd charset the same, 3rd different

		.EQU VDP_CHARSET1C	$02
		.EQU VDP_COLSET1C	$FD

		.EQU VDP_CHARSET3	$03
		.EQU VDP_COLSET3	$FF

		; GR I - Single charset
		;.EQU VDP_GRTYPE	$04
		;.EQU VDP_GRMODE	$00
		;.EQU VDP_PATTAB	$0000	; - $07ff
					; $0800 - $17ff free
		;.EQU VDP_NAMTAB	$1800	; - $1aff
		;.EQU VDP_SPRATR	$1b00	; - $1b7f
					; $1b80 - $1fff free
		;.EQU VDP_COLTAB	$2000	; - $27ff
					; $2800 - $37ff free
		;.EQU VDP_SPRPAT	$3800	; - $3fff

		; GR II - Three charsets
		;.EQU VDP_GRTYPE	$04
		;.EQU VDP_GRMODE	$03
		;.EQU VDP_PATTAB	$0000	; - $17ff
		;.EQU VDP_NAMTAB	$1800	; - $1aff
		;.EQU VDP_SPRATR	$1B00	; - $1b7f
		;.EQU VDP_COLTAB	$2000	; - $37ff
		;.EQU VDP_SPRPAT	$3800	; - $3fff	

;**************************************************************
; Colours

		.EQU VDP_CLR_TRANSPARENT	0
		.EQU VDP_CLR_BLACK		1
		.EQU VDP_CLR_MD_GREEN		2
		.EQU VDP_CLR_LT_GREEN		3
		.EQU VDP_CLR_DK_BLUE		4
		.EQU VDP_CLR_LT_BLUE		5
		.EQU VDP_CLR_DK_RED		6
		.EQU VDP_CLR_CYAN		7
		.EQU VDP_CLR_MD_RED		8
		.EQU VDP_CLR_LT_RED		9
		.EQU VDP_CLR_DK_YELLOW		10
		.EQU VDP_CLR_LT_YELLOW		11
		.EQU VDP_CLR_DK_GREEN		12
		.EQU VDP_CLR_MAGENTA		13
		.EQU VDP_CLR_GREY		14
		.EQU VDP_CLR_WHITE		15

; Name:		VDP_INIT
; Function:	Writes VDP registers from following labels:
;		VDP_GRTYPE
;		VDP_NAMTAB
;		VDP_COLTAB
;		VDP_PATTAB0
;		VDP_SPRATR
;		VDP_SPRPAT
; On Entry:	None
; On Exit:	None
; Destroys:	BC, HL, A

; Name:		VDP_INT
; Function:	Setup VDP Interrupts
; On Entry:	HL = Interrupt routine address
; On Exit:	None
; Destroys:	A

; Name:		VDP_CLCADDR
; Function:	Calculate NAMTAB address
; On Entry:	H = Column, L = Row
; On Exit:	HL = NAMTAB address
; Destroys:	A, DE

; Name:		VDP_WRADDR
; Function:	Set VDP write address
; On Entry:	HL = Address
; On Exit:	None
; Destroys:	None

; Name:		VDP_RDADDR
; Function:	Set VDP read address
; On Entry:	HL = address
; On Exit:	None
; Destroys:	None

; Name:		VDP_FILL
; Function:	Fill VDP ram
; On Entry:	HL = VDP address. BC = count. A = value.
; On Exit:	None
; Destroys:	BC

; Name:		VDP_CLRNAM
; Function:	Clears NAMTAB memory
; On Entry:	None
; On Exit:	None
; Destroys:	HL, BC, A

; Name:		VDP_INITGR
; Function:	Setup NAMTAB for bitmap screen.
; On Entry:	None
; On Exit:	None
; Destroys:	HL, BC, A

; Name:		VDP_SPRLDIR
; Function:	Write sprite pattern data to VDP
; On Entry:	HL = pattern data
; On Exit:	None
; Destroys:	BC, DE, HL, A

; Name:		VDP_PATBANK0
; Function:	Write char pattern data to VDP bank 0
; On Entry:	HL = pattern data
; On Exit:	None
; Destroys:	BC, DE, HL, A

; Name:		VDP_PATBANK1
; Function:	Write char pattern data to VDP bank 1
; On Entry:	HL = pattern data
; On Exit:	None
; Destroys:	BC, DE, HL, A

; Name:		VDP_PATBANK2
; Function:	Write char pattern data to VDP bank 2
; On Entry:	HL = pattern data
; On Exit:	None
; Destroys:	BC, DE, HL, A

; Name:		VDP_LDIR
; Function:	Write data to VDP from memory(HL) to VDP(DE) for bc bytes
; On Entry:	HL = Memory address. DE = VDP Address. BD = Count
; On Exit:	None
; Destroys:	BC, DE, HL, A

; Name:		VDP_SETFONT
; Function:	Write Font pattern and colour data to VDP
; On Entry:	HL = pattern data. BC = Count
; On Exit:	None
; Destroys:	BC, DE, HL, A

; Name:		VDP_WRITE_FF
; Function:	Write bytes terminated with $ff
; On Entry:	DE = Memory address.
; On Exit:	None
; Destroys:	DE, A

; Name:		VDP_SPROFF
; Function:	Turn all sprites off
; On Entry:	None
; On Exit:	None
; Destroys:	HL, A

; Name:		VDP_OUTSTR
; Function:	Write sprite attributes, handling bleeding in from borders
; On Entry:	(VDP_SPR_CNT) = No of sprites to output
; On Exit:	None
; Destroys:	BC, DE, HL, A

; Name:		VDP_SETSPRATR
; Function:	Set sprite Attribute pointer
; On Entry:	HL = Address of sprite attribute.
; On Exit:	None
; Destroys:	None

.MACRO VDP_SETFONTDATA
		ld hl, \1
		ld bc, \2
		call VDP_SETFONT
.ENDM

.MACRO VDP_SETSPRDATA
		ld hl, \1
		ld bc, \2
		call VDP_SPRLDIR
.ENDM

.MACRO VDP_SETSPRATR
		ld hl, \1
		call VDP_SETSPRATR
.ENDM

.MACRO VDP_SETVDPINT
		ld hl, \1
		call VDP_INT
.ENDM

.MACRO VDP_SETNAMPOS_W
		ld hl, (\1 * 32) + \2 + VDP_NAMTAB
		call VDP_WRADDR
.ENDM

.MACRO VDP_SETNAMPOS_R
		ld hl, (\1 * 32) + \2 + VDP_NAMTAB
		call VDP_RDADDR
.ENDM

.MACRO VDP_WRITE_A
		out (VDP_PORT0), a
.ENDM

.MACRO VDP_WRITE_N
		ld a, \1
		out (VDP_PORT0), a
.ENDM

.MACRO VDP_WRITE_STR
		ld hl, (\1 * 32) + \2 + VDP_NAMTAB
		call VDP_WRADDR
		ld de, \3
		call VDP_WRITE_FF
.ENDM

.MACRO VDP_READ_A
		in a, (VDP_PORT0)
.ENDM

.MACRO VDP_FILL_A
		ld hl, \1
		ld bc, \2
		ld a, \3
		call VDP_FILL
.ENDM

		.ENDIF

.MACRO VDP_SETREGS
		.EQU VDP_GRTYPE		\1	;   Graphic mode
		.EQU VDP_GRCHAR		\2	;   Char Sets
		.EQU VDP_GRCOLR		\3	;
		.EQU VDP_SPRMAG		\4
		.EQU VDP_SPRSIZ		\5
		.EQU VDP_BORDER		\6

		.EQU VDP_PATTAB0  \7	; - $07ff
					; $0800   - $17ff free
		.EQU VDP_PATTAB1  \7 + $0800
		.EQU VDP_PATTAB2  \7 + $1000
		.EQU VDP_NAMTAB   \8	; - $1aff
		.EQU VDP_SPRATR   \9	; - $1b7f
					; $1b80   - $1fff free
		.EQU VDP_COLTAB   \10	; - $27ff
					; $2800   - $37ff free
		.EQU VDP_SPRPAT   \11	; - $3fff

		.EXPORT VDP_GRTYPE, VDP_GRCHAR, VDP_GRCOLR
		.EXPORT VDP_SPRMAG, VDP_SPRSIZ, VDP_BORDER
		.EXPORT VDP_PATTAB0, VDP_PATTAB1, VDP_PATTAB2
		.EXPORT VDP_NAMTAB, VDP_SPRATR, VDP_COLTAB, VDP_SPRPAT
.ENDM

